@@ -9,10 +9,10 @@ My::Album->columns(Essential => qw(id title description genre artist));
 
 My::Album->has_a( artist => 'My::Artist' );
 My::Album->has_a( genre => 'My::Genre' );
-My::Album->has_many( tracks => 'My::Track');
+My::Album->has_many( tracks => 'My::Track', {order_by => 'position'} );
 
 sub class_title { 'Album' }
 sub class_plural { 'Albums' }
-sub class_description { 'Class::DBI::Factory is a simple skeleton around which to build web applications.' }
+sub class_description { 'The album class.' }
 
 1;
